/*
 * Copyright (c) 2019 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import Shotcut.Controls 1.0

KeyframableFilter {
    property string corner1x: '0'
    property string corner1y: '1'
    property string corner2x: '2'
    property string corner2y: '3'
    property string corner3x: '4'
    property string corner3y: '5'
    property string corner4x: '6'
    property string corner4y: '7'
    property string stretchx: '8'
    property string stretchy: '9'
    property string interpolator: '10'
    property string featheralpha: '11'
    property string alphaoperation: '12'
    property string enablestretch: '13'
    property string transparentbackground: '14'
        
    property double corner1xDefault: 0.3333
    property double corner1yDefault: 0.3333
    property double corner2xDefault: 0.6666
    property double corner2yDefault: 0.3333
    property double corner3xDefault: 0.6666
    property double corner3yDefault: 0.6666
    property double corner4xDefault: 0.3333
    property double corner4yDefault: 0.6666
    property double stretchxDefault: 0.500
    property double stretchyDefault: 0.500
    property double interpolatorDefault: 0.166
    property double featheralphaDefault: 0.01
    property double alphaoperationDefault: 0.00
         
    keyframableParameters: [corner1x, corner1y, corner2x, corner2y, corner3x, corner3y, corner4x, corner4y, stretchx, stretchy, interpolator, featheralpha, alphaoperation]
    startValues: [0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5]
    middleValues: [corner1xDefault, corner1yDefault, corner2xDefault, corner2yDefault, corner3xDefault, corner3yDefault, corner4xDefault, corner4yDefault, stretchxDefault, stretchyDefault, interpolatorDefault, featheralphaDefault, alphaoperationDefault]
    endValues: [0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5]

    width: 350
    height: 100

    Component.onCompleted: {
        if (filter.isNew) {
            filter.set(corner1x, corner1xDefault)
            filter.set(corner1y, corner1yDefault)
            filter.set(corner2x, corner2xDefault)
            filter.set(corner2y, corner2yDefault)
            filter.set(corner3x, corner3xDefault)
            filter.set(corner3y, corner3yDefault)
            filter.set(corner4x, corner4xDefault)
            filter.set(corner4y, corner4yDefault)
            filter.set(stretchx, stretchxDefault)
            filter.set(stretchy, stretchyDefault)
            filter.set(interpolator, interpolatorDefault)
            filter.set(featheralpha, featheralphaDefault)
            filter.set(alphaoperation, alphaoperationDefault)
            filter.set(enablestretch, 0)
            filter.set(transparentbackground, 0)
            filter.savePreset(preset.parameters)
        }
        setControls()
    }

    function setControls() {
        var position = getPosition()
        blockUpdate = true
        corner1xSlider.value = filter.getDouble(corner1x, position) * corner1xSlider.maximumValue
        corner1ySlider.value = filter.getDouble(corner1y, position) * corner1ySlider.maximumValue
        corner2xSlider.value = filter.getDouble(corner2x, position) * corner2xSlider.maximumValue
        corner2ySlider.value = filter.getDouble(corner2y, position) * corner2ySlider.maximumValue
        corner3xSlider.value = filter.getDouble(corner3x, position) * corner3xSlider.maximumValue
        corner3ySlider.value = filter.getDouble(corner3y, position) * corner3ySlider.maximumValue
        corner4xSlider.value = filter.getDouble(corner4x, position) * corner4xSlider.maximumValue
        corner4ySlider.value = filter.getDouble(corner4y, position) * corner4ySlider.maximumValue
        stretchxSlider.value = filter.getDouble(stretchx, position) * stretchxSlider.maximumValue
        stretchySlider.value = filter.getDouble(stretchy, position) * stretchySlider.maximumValue
        interpolatorSlider.value = filter.getDouble(interpolator, position) * interpolatorSlider.maximumValue
        featheralphaSlider.value = filter.getDouble(featheralpha, position) * featheralphaSlider.maximumValue
        alphaoperationSlider.value = filter.getDouble(alphaoperation, position) * alphaoperationSlider.maximumValue
        blockUpdate = false
        enableControls(isSimpleKeyframesActive())
    }

    function enableControls(enabled) {
        corner1xSlider.enabled = corner1ySlider.enabled = corner2xSlider.enabled = corner2ySlider.enabled = corner3xSlider.enabled = corner3ySlider.enabled = corner4xSlider.enabled = corner4ySlider.enabled = stretchxSlider.enabled = stretchySlider.enabled = interpolatorSlider.enabled = featheralphaSlider.enabled = alphaoperationSlider.enabled = enabled
    }

    function updateSimpleKeyframes() {
        updateFilter(corner1x, corner1xSlider.value / corner1xSlider.maximumValue, corner1xKeyframesButton)
        updateFilter(corner1y, corner1ySlider.value / corner1ySlider.maximumValue, corner1yKeyframesButton)
        updateFilter(corner2x, corner2xSlider.value / corner2xSlider.maximumValue, corner2xKeyframesButton)
        updateFilter(corner2y, corner2ySlider.value / corner2ySlider.maximumValue, corner2yKeyframesButton)
        updateFilter(corner3x, corner3xSlider.value / corner3xSlider.maximumValue, corner3xKeyframesButton)
        updateFilter(corner3y, corner3ySlider.value / corner3ySlider.maximumValue, corner3yKeyframesButton)
        updateFilter(corner4x, corner4xSlider.value / corner4xSlider.maximumValue, corner4xKeyframesButton)
        updateFilter(corner4y, corner4ySlider.value / corner4ySlider.maximumValue, corner4yKeyframesButton)
        updateFilter(stretchx, stretchxSlider.value / stretchxSlider.maximumValue, stretchxKeyframesButton)
        updateFilter(stretchy, stretchySlider.value / stretchySlider.maximumValue, stretchyKeyframesButton)
        updateFilter(interpolator, interpolatorSlider.value / interpolatorSlider.maximumValue, interpolatorKeyframesButton)
        updateFilter(featheralpha, featheralphaSlider.value / featheralphaSlider.maximumValue, featheralphaKeyframesButton)
        updateFilter(alphaoperation, alphaoperationSlider.value / alphaoperationSlider.maximumValue, alphaoperationKeyframesButton)
    }

    GridLayout {
        anchors.fill: parent
        anchors.margins: 8
        columns: 4

        Label {
            text: qsTr('Preset')
            Layout.alignment: Qt.AlignRight
        }
        Preset {
            id: preset
            parameters: [corner1x, corner1y, corner2x, corner2y, corner3x, corner3y, corner4x, corner4y, stretchx, stretchy, interpolator, featheralpha, alphaoperation]
            Layout.columnSpan: 3
            onBeforePresetLoaded: {
                filter.resetProperty(corner1x)
                filter.resetProperty(corner1y)
                filter.resetProperty(corner2x)
                filter.resetProperty(corner2y)
                filter.resetProperty(corner3x)
                filter.resetProperty(corner3y)
                filter.resetProperty(corner4x)
                filter.resetProperty(corner4y)
                filter.resetProperty(stretchx)
                filter.resetProperty(stretchy)
                filter.resetProperty(interpolator)
                filter.resetProperty(featheralpha)
                filter.resetProperty(alphaoperation)
                filter.resetProperty(enablestretch)
                filter.resetProperty(transparentbackground)
                
                
            }
            onPresetSelected: {
                setControls()
                initializeSimpleKeyframes()
            }
        }

        Label {
            text: qsTr('Corner 1 X')
            Layout.alignment: Qt.AlignRight
                   
        }
        SliderSpinner {
            id: corner1xSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner1x, corner1xSlider.value / corner1xSlider.maximumValue, corner1xKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner1xSlider.value = corner1xDefault * corner1xSlider.maximumValue
        }
        KeyframesButton {
            id: corner1xKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner1x) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner1x, corner1xSlider.value / corner1xSlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Corner 1 Y')
            Layout.alignment: Qt.AlignRight
                       
        }
        SliderSpinner {
            id: corner1ySlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner1y, corner1ySlider.value / corner1ySlider.maximumValue, corner1yKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner1ySlider.value = corner1yDefault * corner1ySlider.maximumValue
        }
        KeyframesButton {
            id: corner1yKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner1y) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner1y, corner1ySlider.value / corner1ySlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Corner 2 X')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: corner2xSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner2x, corner2xSlider.value / corner2xSlider.maximumValue, corner2xKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner2xSlider.value = corner2xDefault * corner2xSlider.maximumValue
        }
        KeyframesButton {
            id: corner2xKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner2x) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner2x, corner2xSlider.value / corner2xSlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Corner 2 Y')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: corner2ySlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner2y, corner2ySlider.value / corner2ySlider.maximumValue, corner2yKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner2ySlider.value = corner2yDefault * corner2ySlider.maximumValue
        }
        KeyframesButton {
            id: corner2yKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner2y) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner2y, corner2ySlider.value / corner2ySlider.maximumValue)
            }
        }
 
 Label {
            text: qsTr('Corner 3 X')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: corner3xSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner3x, corner3xSlider.value / corner3xSlider.maximumValue, corner3xKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner3xSlider.value = corner3xDefault * corner3xSlider.maximumValue
        }
        KeyframesButton {
            id: corner3xKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner3x) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner3x, corner3xSlider.value / corner3xSlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Corner 3 Y')
            Layout.alignment: Qt.AlignRight
                     
        }
        SliderSpinner {
            id: corner3ySlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner3y, corner3ySlider.value / corner3ySlider.maximumValue, corner3yKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner3ySlider.value = corner3yDefault * corner3ySlider.maximumValue
        }
        KeyframesButton {
            id: corner3yKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner3y) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner3y, corner3ySlider.value / corner3ySlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Corner 4 X')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: corner4xSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner4x, corner4xSlider.value / corner4xSlider.maximumValue, corner4xKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner4xSlider.value = corner4xDefault * corner4xSlider.maximumValue
        }
        KeyframesButton {
            id: corner4xKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner4x) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner4x, corner4xSlider.value / corner4xSlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Corner 4 Y')
            Layout.alignment: Qt.AlignRight
                       
        }
        SliderSpinner {
            id: corner4ySlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(corner4y, corner4ySlider.value / corner4ySlider.maximumValue, corner4yKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: corner4ySlider.value = corner4yDefault * corner4ySlider.maximumValue
        }
        KeyframesButton {
            id: corner4yKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(corner4y) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, corner4y, corner4ySlider.value / corner4ySlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Stretch X')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: stretchxSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(stretchx, stretchxSlider.value / stretchxSlider.maximumValue, stretchxKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: stretchxSlider.value = stretchxDefault * stretchxSlider.maximumValue
        }
        KeyframesButton {
            id: stretchxKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(stretchx) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, stretchx, stretchxSlider.value / stretchxSlider.maximumValue)
            }
        }
 
        Label {
            text: qsTr('Stretch Y')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: stretchySlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(stretchy, stretchySlider.value / stretchySlider.maximumValue, stretchyKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: stretchySlider.value = stretchyDefault * stretchySlider.maximumValue
        }
        KeyframesButton {
            id: stretchyKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(stretchy) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, stretchy, stretchySlider.value / stretchySlider.maximumValue)
            }
        }
 
        Label {
            text: qsTr('Interpolator')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: interpolatorSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(interpolator, interpolatorSlider.value / interpolatorSlider.maximumValue, interpolatorKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: interpolatorSlider.value = interpolatorDefault * interpolatorSlider.maximumValue
        }
        KeyframesButton {
            id: interpolatorKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(interpolator) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, interpolator, interpolatorSlider.value / interpolatorSlider.maximumValue)
            }
        }       
  
       Label {
            text: qsTr('Feather Alpha')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: featheralphaSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(featheralpha, featheralphaSlider.value / featheralphaSlider.maximumValue, featheralphaKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: featheralphaSlider.value = featheralphaDefault * featheralphaSlider.maximumValue
        }
        KeyframesButton {
            id: featheralphaKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(featheralpha) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, featheralpha, featheralphaSlider.value / featheralphaSlider.maximumValue)
            }
        }       
        
        Label {
            text: qsTr('Alpha Operation')
            Layout.alignment: Qt.AlignRight
                        
        }
        SliderSpinner {
            id: alphaoperationSlider
            minimumValue: 0
            maximumValue: 100
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(alphaoperation, alphaoperationSlider.value / alphaoperationSlider.maximumValue, alphaoperationKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: alphaoperationSlider.value = alphaoperationDefault * alphaoperationSlider.maximumValue
        }
        KeyframesButton {
            id: alphaoperationKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(alphaoperation) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, alphaoperation, alphaoperationSlider.value / alphaoperationSlider.maximumValue)
            }
        }                    

Label {}
        CheckBox {
            id: enablestretchCheckBox
            text: qsTr('Enable Stretch')
            Layout.columnSpan: 2
            checked: filter.get('enablestretch') === '1'
            property bool isReady: false
            Component.onCompleted: isReady = true
            onClicked: {
                if (isReady)
                    filter.set('enablestretch', checked)
            }
        }
        
Label {}
        CheckBox {
            id: transparentbackgroundCheckBox
            text: qsTr('Transparent Background')
            Layout.columnSpan: 2
            checked: filter.get('transparentbackground') === '1'
            property bool isReady: false
            Component.onCompleted: isReady = true
            onClicked: {
                if (isReady)
                    filter.set('transparentbackground', checked)
            }
        }
                                            
        Item {
            Layout.fillHeight: true
        }
    }

    Connections {
        target: filter
        onInChanged: updateSimpleKeyframes()
        onOutChanged: updateSimpleKeyframes()
        onAnimateInChanged: updateSimpleKeyframes()
        onAnimateOutChanged: updateSimpleKeyframes()
    }

    Connections {
        target: producer
        onPositionChanged: setControls()
    }
}
